/*****************************************************************************
 * Copyright ? 2015, SAS Institute Inc., Cary, NC, USA.  All Rights Reserved.
 ****************************************************************************/

// NOTE: Requires Service Architecture runtime and will not function standalone

@BaseScript(MasterETLBaseScript)

import groovy.transform.BaseScript

exitJVM = true

// Set up expected CLI syntax usage:
def cli = new CliBuilder(
    usage:  "removeEMIContent [-vdh]",
    header: "\nRemove metadata folders associated with the SAS Environment Manager " +
            "Service Architecture.\n" +
            "Options:",
    footer: "Exit status:\n" +
            "$RC_SUCCESS  if OK\n" +
            "$RC_WARNING  if minor issues are encountered\n" +
            "$RC_GENERAL_ERROR  if an error is encountered\n" +
            "$RC_INVALID_USAGE for invalid usage"
)
cli.with {
    v(longOpt:'verbose', 'Enable verbose output', required: false)
    d(longOpt:'debug', 'Enable debug output', required: false)
    h(longOpt:'help', 'Show usage information', required: false)
}

// Parse given arguments
OptionAccessor opt = cli.parse(args)
if (!opt) {
    term.error "Invalid argument(s): $args"
    exitPgm(RC_INVALID_USAGE)
}
if (opt.h) {
    cli.usage()
    exitPgm(RC_SUCCESS)
}
if (opt.v) {
    verbose = true
}
if (opt.d) {
    debug = true
    evDebug = 2
    setLogLevel LogLevel.DEBUG
}

if ( !frameworkInitialized ) {
    msg.info "The SAS Environment Manager Service Architecture is not initialized; exiting."    
    return exitPgm(RC_SUCCESS)
}

String emiLogDir     = sysProps['emi.log.dir']
String sasHomeDir    = sysProps['sas.home']
String emiConfigDir  = sysProps['sasev.configDir']


msg.info "Service Architecture Metadata Folder Removal Utility - ${new Date()}"
      
// Step 1 Back-up Metadata Content
term.info ""   
msg.info "Preparing to back up existing SAS metadata content..."
    
File pkgDir = canonicalFile(frameworkDir,'SASPackages');    
File emiContentFile = canonicalFile(pkgDir, "productsFolderBackup.${_LOG_TIMESTAMP}.spk")

if (!emiContentFile.exists()){
    
    msg.info "\tBacking up existing SAS metadata content"
        
    Map myOtherOpts=[:]
    myOtherOpts.env= [JAVA_TOOL_OPTIONS:'-Dsas.promotion.stp.includesourcecode=false']
    
    cmd = [
        "${sasHomeDir}/SASPlatformObjectFramework/9.4/ExportPackage",
        "-host", "${hq.connectionInfo.metadatahost}",
        "-port", "${hq.connectionInfo.metadataport}",
        "-user", "${hq.connectionInfo.user}", "-password", "${hq.connectionInfo.password}",
        "-log", "${emiLogDir}/remove_emiContent_backup.${_LOG_TIMESTAMP}.log",
        "-package", "$emiContentFile",
        "-subprop",
        "-disableX11",           
        "-objects", "/Products/SAS Environment Manager"
    ]
    
    if (verbose) { msg.info "\tExporting existing SAS metadata content to [$emiContentFile]" }
    
    log.info "\tMetadata export command:"
    cmd.each { arg ->
        if (arg != "${hq.connectionInfo.password}") {
            log.info "\t  $arg"
        } else {
            log.info "\t  ********"
        }
    }
    
    int rc = exec(myOtherOpts, cmd)
    if (rc != 0) {        
        msg.error "\tError exporting package $emiContentFile.name rc = [$rc]"
        msg.error "\tCheck $logFile.name, then rerun this script"
        return exitPgm(RC_GENERAL_ERROR)
    }
} else {
    msg.info "$emiContentFile already exists; skipping export of metadata content"
}
    
// Step 2 Delete metadata content   
term.info ""
msg.info "Deleting existing SAS Environment Manager metadata folder and contents..."
cmd = [
    "${sasHomeDir}/SASPlatformObjectFramework/9.4/tools/sas-delete-objects",
    "-host", "${hq.connectionInfo.metadatahost}",
    "-port", "${hq.connectionInfo.metadataport}",
    "-user", "${hq.connectionInfo.user}", "-password", "${hq.connectionInfo.password}",
    "-log", "${emiLogDir}/remove_emiContent_delete.${_LOG_TIMESTAMP}.log",
    "/Products/SAS Environment Manager",
    "-deleteContents"
]
    
(verbose? msg:log).info "\tRemoving existing SAS metadata folder /Products/SAS Environment Manager and contents" 
    
log.info "\tMetadata delete command:"
cmd.each { arg ->
    if (arg != "${hq.connectionInfo.password}") {
        log.info "\t  $arg"
    } else {
        log.info "\t  ********"
    }
}
    
rc = exec(cmd)
if (rc != 0) {
    msg.error "Error deleting SAS metadata folder /Products/SAS Environment Manager  rc = [$rc]"
    msg.error "Check $logFile.name, then rerun this script"
    return exitPgm(RC_GENERAL_ERROR)
}
        
        
term.info ""
msg.info "The removal of the Service Architecture metadata folders completed successfully"    
return exitPgm(RC_SUCCESS)




def exitPgm(int rc) {
    term.info ""
    msg.info "Exiting Service Architecture Metadata Folder Removal Utility"
    
    if (rc == RC_GENERAL_ERROR){             
        msg.info "The removal of the Service Architecture metadata folders did NOT complete successfully"
        msg.warn "Check $logFile.name for details, correct any issues and rerun this script"
    }

    if (exitJVM) {
        System.exit rc
    } else {
        return rc
    }
}
